/*
    Copyright (c) 2013 aizo ag, Zurich, Switzerland

    This file is part of libdsuid.

    libdsuid is free software: you can redistribute it and/or modify
    it under the terms of the GNU General Public License as published by
    the Free Software Foundation, either version 3 of the License, or
    (at your option) any later version.

    libdsuid is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with digitalSTROM Server. If not, see <http://www.gnu.org/licenses/>.
*/

#ifndef __LIBDSUID_H__
#define __LIBDSUID_H__

#include <stdbool.h>
#include <stdint.h>

#ifdef __cplusplus
extern "C" {
#endif

/* TODO: remove those undef', once they have been removed from ds485-core */
#undef DSUID_SIZE
#undef DSUID_STR_LEN
#undef DSUID_STR

#define DSUID_SIZE      17
#define DSUID_STR_LEN   ((2 * DSUID_SIZE) + 1)
#define DSUID_STR(name) char name[DSUID_STR_LEN]

#define DSUID_DEFINE(name,u0,u1,u2,u3,u4,u5,u6,u7,u8,u9,u10,u11,u12,u13,u14, \
                     u15,u16) \
  static const dsuid_t name __attribute__ ((unused)) = \
    {{u0,u1,u2,u3,u4,u5,u6,u7,u8,u9,u10,u11,u12,u13,u14,u15,u16}}

#ifndef __MOVE_DSUID_H__
#define __MOVE_DSUID_H__

/**
 * 136-Bit dSUID
 */
typedef struct {
    unsigned char id[DSUID_SIZE];
#ifdef SWIG
} dsuid_t;
#else
} __attribute__ ((packed)) dsuid_t;
#endif

/**
 * 96-Bit dSID
 */
typedef struct dsid {
    unsigned char id[12];
} dsid_t;

#endif /* __MOVE_DSUID_H__ */

DSUID_DEFINE(DSUID_BROADCAST, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, \
             0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff, 0xff);
DSUID_DEFINE(DSUID_NULL, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, \
             0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00, 0x00);

/* return codes */
typedef enum {
    DSUID_RC_OK         = 0,  /* success                  */
    DSUID_RC_INV_DSUID  = 1,  /* dsuid can not be used    */
    DSUID_RC_INV_SERIAL = 2,  /* unexpected serial number */
    DSUID_RC_GEN_FAIL   = 3,  /* general system failure   */
    DSUID_RC_WRONG_PAR  = 4,  /* wrong function paramater */
    DSUID_RC_UUID_ERR   = 5,  /* error from ossp uuid lib */
} dsuid_rc_t;

const char *dsuid_strerror(int err);
const char *dsuid_rc_to_text(int rc) __attribute__ ((deprecated));

/* namespaces */
typedef enum {
    DSUID_NS_ENOCEAN,
    DSUID_NS_GS1_128,
    DSUID_NS_IEEE_MAC,
    DSUID_NS_VDSM,
    DSUID_NS_VDC,
} dsuid_ns_t;


int dsuid_generate_v1(dsuid_t *dsuid);
int dsuid_generate_v1_mac(const char *mac, dsuid_t *dsuid);
int dsuid_generate_v3_from_namespace(dsuid_ns_t ns, const char *name, dsuid_t *dsuid);
int dsuid_generate_v4_random(dsuid_t *dsuid);

dsuid_t dsuid_from_dsid(const dsid_t *dsid);
int dsuid_to_dsid(const dsuid_t *dsuid, dsid_t *dsid);
int dsuid_get_next_dsuid(const dsuid_t *dsuid, dsuid_t *dsuid_next);
int dsuid_get_previous_dsuid(const dsuid_t *dsuid, dsuid_t *dsuid_prev);
int dsuid_get_main_dsuid(const dsuid_t *dsuid, const uint8_t module_count, dsuid_t *dsuid_main);
int dsuid_get_serial_number(const dsuid_t *dsuid, uint32_t *serialNumber);

int dsuid_from_string(const char *buf, dsuid_t *dsuid);
int dsuid_to_string(const dsuid_t *dsuid, char *buf);

/**
 * strdsuid returns dsuid as printable string in reusable thread-local buffer
 * @param  dsuid  to be printed
 * @return        pointer to thread local buffer
 *
 * printf("%s", strdsuid(dsuid));  #works
 *
 * CAUTION: accidental rewrite of buffer
 * printf("%s %s", strdsuid(dsuid), strdsuid(dsuid2));  #doesn't work
 */
const char *strdsuid(const dsuid_t *dsuid);

bool dsuid_equal(const dsuid_t *dsuid1, const dsuid_t *dsuid2);
bool dsuid_is_null(const dsuid_t *dsuid);
bool dsuid_is_broadcast(const dsuid_t *dsuid);

#ifdef __cplusplus
}
#endif

#endif
